@@ -1,20 +1,19 @@
 from typing import Iterator, Any
 
-from .tokenizer import Token, tokenize, detokenize, split_sentences
+from .tokenizer import Token, tokenize, detokenize, split_sentences, split_sentences_old
 from .normalizer import normalize, normalize_sentence
 from .inverse_normalizer import inverse_normalize_sentence, inverse_normalize_timecoded
 from .utils import (
-    strip_punct, filter_out_chars, capitalize, pre_process,
+    strip_punct, filter_out_chars, filter_in_chars, capitalize, pre_process,
     extract_parenthesis_content, sentence_stats,
 )
-from .definitions import PUNCTUATION
+from .definitions import PUNCTUATION, LETTERS, PUNCT_PAIRS, VALID_CHARS
 from ..utils import read_file_drop_comments
 
 
-
 def load_translation_dict(path: str) -> dict:
     translation_dict = dict()
-    with open(path, 'r') as f:
+    with open(path, 'r', encoding='utf-8') as f:
         for line in f.readlines():
             line = line.strip()
             if '\t' in line:
@@ -27,6 +26,8 @@ def load_translation_dict(path: str) -> dict:
 
 
 def reverse_translation_dict(path: str, newpath: str) -> None:
+    """ Build a translation dictionary (tsv file) by reversing another translation dictionary
+    """
     reversed = dict()
     for line in read_file_drop_comments(path):
         line = pre_process(line)
@@ -39,7 +40,7 @@ def reverse_translation_dict(path: str, newpath: str) -> None:
                 reversed[val] += ", {}".format(key)
             else:
                 reversed[val] = key
-    with open(newpath, 'w') as f:
+    with open(newpath, 'w', encoding='utf-8') as f:
         for k in sorted(reversed):
             f.write(f"{k}\t{reversed[k]}\n")
 
@@ -50,7 +51,7 @@ def correct_sentence(sentence: str) -> str:
 
 
 
-def translate(token_stream: Iterator[Token], tra_dict: dict, **options: Any) -> Iterator[Token]:
+def translate_tokens(token_stream: Iterator[Token], tra_dict: dict, **options: Any) -> Iterator[Token]:
     """ Substitute tokens according to a given dictionary
         
         Keys with uppercase letters will be case-sentitive
@@ -94,3 +95,40 @@ def translate(token_stream: Iterator[Token], tra_dict: dict, **options: Any) ->
                     tok.data = tra_dict[tok.data]
                     break
         yield tok
+
+
+def count_words(sentence: str) -> int:
+    """ Return number of regular words in sentence """
+    n = 0
+    for t in tokenize(sentence, norm_punct=True, autocorrect=True):
+        if t.kind == Token.WORD:
+            n += 1
+    return n
+
+
+def is_full_sentence(sentence: str) -> bool:
+    """Check if sentence is a complete sentence, punctuation-wise"""
+    return sentence[0].isupper() and sentence[-1] in ".!?…"
+
+def is_sentence_start_open(sentence: str) -> bool:
+    return (
+        sentence[0].islower()
+        or (sentence[0].isupper() and (sentence[1].isupper() or sentence[1].isdigit()))  # Acronym
+        or sentence[0] in "'’"
+        or sentence[0].isdigit()
+    )
+	
+def is_sentence_end_open(sentence: str) -> bool:
+    return (
+        sentence[-1].islower()
+        or sentence[-1] in "…'’,»\""
+        or sentence[-1].isdigit()
+    )
+
+def is_sentence_punct_paired(sentence: str) -> bool:
+    if sentence.count('"') % 2 != 0:
+        return False
+    for punct in PUNCT_PAIRS:
+        if sentence.count(punct) != sentence.count(PUNCT_PAIRS[punct]):
+            return False
+    return True
\ No newline at end of file
